import { ProjectActionTypes } from './project.action.type';
import { Action } from '@ngrx/store';
import { MsgState } from '../msg.state';
import { ProjectState } from './project.state';
import { ProjectActionUnion } from './project.action';
import { Project } from 'src/app/shared/models/project/project.model';


const projectInitState: ProjectState = {
  currestProject: null,
  projectsList: [],
  msgState: MsgState.INIT_STATE,
  errorMsg: 'no error'
};

export function projectReducer(state: ProjectState = projectInitState, action: Action): ProjectState{

  switch(action.type){

    //create form to add project
    case ProjectActionTypes.CREATE_FORM_ADD_PROJECT:
      return {
        ...state,
        msgState: MsgState.FORM_LOADING,
      };
    case ProjectActionTypes.CREATE_FORM_ADD_PROJECT_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg:(<ProjectActionUnion> action).payload
      };
    case ProjectActionTypes.CREATE_FORM_ADD_PROJECT_SUCCESS:
      return {
        ...state,
        msgState: MsgState.FORM_LOADED
      };

      //save new project
    case ProjectActionTypes.SAVE_NEW_PROJECT:
      return {
        ...state,
        msgState: MsgState.ADD
      };
    case ProjectActionTypes.SAVE_NEW_PROJECT_SUCCESS:
      let newProject: Project = (<ProjectActionUnion> action).payload;
      let projectsListCopy1: Array<Project> = [...state.projectsList];
      projectsListCopy1.push(newProject);
      return {
        ...state,
        msgState: MsgState.ADDED,
        projectsList: projectsListCopy1,
        currestProject: newProject
      };

    //get all projects
    case ProjectActionTypes.GET_ALL_PROJECTS:
      return {
        ...state,
        msgState: MsgState.LOADING
      };
    case ProjectActionTypes.GET_ALL_PROJECTS_SUCCESS:
      let projectsListCopy2: Array<Project> = (<ProjectActionUnion> action).payload;
      return {
        ...state,
        msgState:MsgState.LOADED,
        projectsList: projectsListCopy2
      };
    case ProjectActionTypes.GET_ALL_PROJECTS_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg:(<ProjectActionUnion> action).payload
      };

     // create form to update project
    case ProjectActionTypes.CREATE_FORM_UPDATE_PROJECT:
      return {
        ...state,
        msgState: MsgState.FORM_LOADING
      };
    case ProjectActionTypes.CREATE_FORM_UPDATE_PROJECT_SUCCCESS:
      let projectToUpdate: Project = (<ProjectActionUnion> action).payload
      return {
        ...state,
        msgState: MsgState.FORM_LOADED,
        currestProject: projectToUpdate
      };
    case ProjectActionTypes.CREATE_FORM_UPDATE_PROJECT_ERROR:
      let err: string = (<ProjectActionUnion> action).payload;
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg:err
      };

    // update project
    case ProjectActionTypes.UPDATE_PROJECT:
      return {
        ...state,
        msgState: MsgState.UPDATE,
      };
    case ProjectActionTypes.UPDATE_PROJECT_SUCCESS:
      let updatedProject : Project = (<ProjectActionUnion> action).payload;
      let projectsListCopy3: Array<Project> = [...state.projectsList];
      projectsListCopy3.map((project: Project)=>{
        project.projectID === updatedProject.projectID?updatedProject:project;
      })
      return {
        ...state,
        msgState:MsgState.UPDATED,
        currestProject: updatedProject,
        projectsList: projectsListCopy3
      };
    case ProjectActionTypes.UPDATE_PROJECT_ERROR:
      let returnedError: string=(<ProjectActionUnion> action).payload;
      return {
        ...state,
        msgState:MsgState.ERR,
        errorMsg:returnedError
      };

    // delete project
    case ProjectActionTypes.DELETE_PROJECT:
      return {
        ...state,
        msgState: MsgState.DELETE
      };
    case ProjectActionTypes.DELETE_PROJECT_SUCCESS:
      let deletedProject: Project =(<ProjectActionUnion> action).payload;
      let indexDeleteProj = state.projectsList.indexOf(deletedProject);
      let projectsListCopy4: Array<Project> =[... state.projectsList];
      projectsListCopy4 = projectsListCopy4.splice(indexDeleteProj,1);
      return {
        ...state,
        msgState: MsgState.DELETED,
        currestProject: deletedProject,
        projectsList: projectsListCopy4
      };
    case ProjectActionTypes.DELETE_PROJECT_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        errorMsg: (<ProjectActionUnion> action).payload
      };
    default:
      return {...state};

    // get projects assigned to user ID

  }
}
