import { catchError } from 'rxjs/operators';
import { AddressActionTypes } from './address.action.type';
import {
  AddressActionUnion,
  GetAllAddressesActionSuccess,
  GetAllAddressesActionError,
  CreateFormAddAddressActionSuccess,
  CreateFormAddAddressActionError,
  SaveNewAddressActionSuccess,
  SaveNewAddressActionError,
  CreateFormUpdateAddressActionSuccess,
  CreateFormUpdateAddressActionError,
  UpdateAddressActionSuccess,
  UpdateAddressActionError,
  DeleteAddressActionError,
  DeleteAddressActionSuccess,
} from './address.action';
import { Observable, mergeMap, map, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { AddressService } from '../../shared/services/address.service';
import { Injectable } from '@angular/core';
import { Address } from 'src/app/shared/models/user/address.model';

@Injectable()
export class AddressEffects {
  constructor(
    private addressService: AddressService,
    private addressActions: Actions
  ) {}

  getAllAddressEffect: Observable<AddressActionUnion> = createEffect(() =>
    this.addressActions.pipe(
      ofType(AddressActionTypes.GET_ALL_ADDRESSES),
      mergeMap((action: AddressActionUnion) => {
        return this.addressService.getAllAddressesService().pipe(
          map(
            (addresses: Array<Address>) =>
              new GetAllAddressesActionSuccess(addresses)
          ),
          catchError((err) => of(new GetAllAddressesActionError(err)))
        );
      })
    )
  );

  createFormAddAddressEffect : Observable<AddressActionUnion> = createEffect(
    ()=> this.addressActions.pipe(
      ofType(AddressActionTypes.CREATE_FORM_ADD_ADDRESS),
      map((action: AddressActionUnion) => new CreateFormAddAddressActionSuccess({})),
      catchError(err => of(new CreateFormAddAddressActionError(err)))

    )
  );

  saveNewAddressEffect: Observable<AddressActionUnion> = createEffect(
    ()=> this.addressActions.pipe(
      ofType(AddressActionTypes.SAVE_NEW_ADDRESS),
      mergeMap((action:AddressActionUnion)=>{
        return this.addressService.createAddressService(action.payload).pipe(
          map((address: Address) => new SaveNewAddressActionSuccess(address)),
          catchError(err => of(new SaveNewAddressActionError(err)))
        )
      })
    )
  );

  createFormUpdateAddressEffect : Observable<AddressActionUnion> = createEffect(
    ()=> this.addressActions.pipe(
      ofType(AddressActionTypes.CREATE_FORM_UPDATE_ADDRESS),
      mergeMap((action: AddressActionUnion)=>{
        return this.addressService.getAddressByIDService(action.payload.addressID).pipe(
          map ((address: Address)=> new CreateFormUpdateAddressActionSuccess(address)),
          catchError(err=> of(new CreateFormUpdateAddressActionError(err)))
        )
      })
    )
  );

  updateAddressEffect : Observable<AddressActionUnion> = createEffect(
    ()=> this.addressActions.pipe(
      ofType(AddressActionTypes.UPDATE_ADDRESS),
      mergeMap( (action: AddressActionUnion)=>{
        return this.addressService.updateAddressService(action.payload).pipe(
          map((address: Address)=> new UpdateAddressActionSuccess(address)),
          catchError(err=> of(new UpdateAddressActionError(err)))
        )
      })
    )
  );

  deleteUserEffect: Observable<AddressActionUnion> = createEffect(() =>
    this.addressActions.pipe(
      ofType(AddressActionTypes.DELETE_ADDRESS),
      mergeMap((action: AddressActionUnion) => {
        return this.addressService.deleteAddressService(action.payload.addressID).pipe(
          map(() => new DeleteAddressActionSuccess(action.payload)),
          catchError((err) => of(new DeleteAddressActionError(err)))
        );
      })
    )
  );

}
