import { catchError, map } from 'rxjs/operators';
import { UserActionTypes } from './user.action.type';
import {
  UserActionUnion,
  CreateFormUpdateUserActionSuccess,
  CreateFormUpdateUserActionError,
  UpdateUserActionSuccess,
  UpdateUserActionError,
} from './user.actions';
import { Observable, mergeMap, of } from 'rxjs';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { UserService } from './../../shared/services/user.service';
import { Injectable } from '@angular/core';
import { User } from 'src/app/shared/models/user/user.model';

@Injectable()
export class UserUpdateEffects {
  constructor(private userService: UserService, private userActions: Actions) {}

  //create form to update a user
  createFormUpdateUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.CREATE_FORM_UPDATE_USER),
      mergeMap((action: UserActionUnion) => {
        return this.userService.getUserByID(action.payload.userID).pipe(
          map((user: User) => new CreateFormUpdateUserActionSuccess(user,{})),
          catchError((err) => of(new CreateFormUpdateUserActionError(err,{})))
        );
      })
    )
  );

  //update user
  updateUserEffect : Observable<UserActionUnion> = createEffect(
    ()=> this.userActions.pipe(
      ofType(UserActionTypes.UPDATE_USER),
      mergeMap((action: UserActionUnion) =>{
        return this.userService.updateUserService(action.payload).pipe(
          map ((user: User) => new UpdateUserActionSuccess(user)),
          catchError( err => of(new UpdateUserActionError(err)))
        )
      })
    )
  )
}
