import { ProjState } from './../../../../shared/models/project/project.state';
import { Project } from 'src/app/shared/models/project/project.model';
import { GetAllComapaniesAction } from './../../../../ngrx/company/company.actions';
import { GetAllUsersAction } from 'src/app/ngrx/user/user.actions';
import { CreateFormUpdatedProjectAction, UpdateProjectAction } from './../../../../ngrx/project/project.action';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ProjectState } from './../../../../ngrx/project/project.state';
import { ProjectService } from './../../../../shared/services/project.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Component, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { UserState } from 'src/app/ngrx/user/user.state';
import { CompanyState } from 'src/app/ngrx/company/company.state';
import { Priority } from 'src/app/shared/models/project/project.priority';
import { MsgState } from 'src/app/ngrx/msg.state';

@Component({
  selector: 'app-project-update',
  templateUrl: './project-update.component.html',
  styleUrls: ['./project-update.component.css'],
})
export class ProjectUpdateComponent implements OnInit {
  projectID: number;

  constructor(
    activatedRoute: ActivatedRoute,
    private projectService: ProjectService,
    private store: Store<{
      projectReducerSelector: ProjectState;
      userReducerSelector: UserState;
      companyReducerSelector: CompanyState;
    }>,
    private formBuilder: FormBuilder,
    private router: Router
  ) {
    this.projectID = activatedRoute.snapshot.params['projectID'];
  }

  userState!: UserState;
  companyState!: CompanyState;

  projectFormGroup!: FormGroup;
  projectState!: ProjectState;
  priorities = Priority;
  projStates = ProjState;
  msgState = MsgState;

  ngOnInit(): void {
    this.store.dispatch(new GetAllUsersAction({}, {}));
    this.store.subscribe((state) => {
      this.userState = state.userReducerSelector;
    });
    this.store.dispatch(new GetAllComapaniesAction({}));
    this.store.subscribe((state) => {
      this.companyState = state.companyReducerSelector;
    });

    this.projectService
      .getProjectByIDService(this.projectID)
      .subscribe((project) => {
        this.store.dispatch(
          new CreateFormUpdatedProjectAction(project, {}, {})
        );
        this.store.subscribe((state) => {
          this.projectState = state.projectReducerSelector;

          this.projectFormGroup = this.formBuilder.group({
            projectID: [this.projectState.currestProject?.projectID],
            projectName: [
              this.projectState.currestProject?.projectName,
              [
                Validators.required,
                Validators.minLength(5),
                Validators.maxLength(20),
              ],
            ],
            priority: [
              this.projectState.currestProject?.priority,
              Validators.required,
            ],
            createdDate: [this.projectState.currestProject?.createdDate],
            description: [
              this.projectState.currestProject?.description,
              [Validators.required, Validators.minLength(25)],
            ],
            projState: [
              this.projectState.currestProject?.projState,
              Validators.required,
            ],
            userID: [
              this.projectState.currestProject?.user.userID,
              Validators.required,
            ],
            companyID: [
              this.projectState.currestProject?.company.companyID,
              Validators.required,
            ],
          });
        });
      });
  }

  onProjectUpdate() {

    let projectFormModel = this.projectFormGroup.value;
    let project : Project = new Project();
    project.projectID = projectFormModel.projectID;
    project.projectName = projectFormModel.projectName;
    project.priority = projectFormModel.priority;
    project.companyID = projectFormModel.companyID;
    project.description = projectFormModel.description;
    project.projState = projectFormModel.projState;
    project.userID = projectFormModel.userID;

    if(window.confirm(this.msgState.CONFIRM_UPDATE)){
      this.store.dispatch(new UpdateProjectAction(project));
    }

  }
  onUpdateSuccess() {
    this.router.navigateByUrl("/projects-management");
  }
}
