import { UserType } from './../../../../shared/models/user/user.type';
import { Router } from '@angular/router';
import { User } from 'src/app/shared/models/user/user.model';
import { MsgState } from './../../../../ngrx/msg.state';
import {
  CreateFormAddUserAction,
  SaveNewUserAction,
} from './../../../../ngrx/user/user.actions';
import { UserState } from './../../../../ngrx/user/user.state';
import { Component, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { GetAllAddressesAction } from 'src/app/ngrx/user-s-address/address.action';
import { AddressState } from 'src/app/ngrx/user-s-address/address.state';

@Component({
  selector: 'app-user-create',
  templateUrl: './user-create.component.html',
  styleUrls: ['./user-create.component.css'],
})
export class UserCreateComponent implements OnInit {
  constructor(
    private store: Store<{
      userReducerSelector: UserState;
      addressReducerSelector: AddressState;
    }>,
    private formBuilder: FormBuilder,
    private router: Router
  ) {}

  userFormGroup!: FormGroup;
  userState!: UserState;
  msgState = MsgState;
  addressState!: AddressState;
  userType = UserType;


  ngOnInit(): void {

    this.store.dispatch(new GetAllAddressesAction({}));
    this.store.subscribe({
      next: (state) => {
        this.addressState = state.addressReducerSelector;
      },
      error:(err)=>{
      }
    });


    this.store.dispatch(new CreateFormAddUserAction({},{}));
    this.store.subscribe({
      next: (state) => {
        this.userState = state.userReducerSelector;

        this.userFormGroup = this.formBuilder.group({

          firstname: [
            '',
            [
              Validators.required,
              Validators.minLength(3),
              Validators.maxLength(15),
            ],
          ],
          lastname: [
            '',
            [
              Validators.required,
              Validators.minLength(3),
              Validators.maxLength(15),
            ],
          ],
          userType: ['', Validators.required],
          addressID:['', Validators.required],
        });
      },
      error: (err) => {
      },
    });
  }

  onUserSave() {
    let model: any = this.userFormGroup.value;
    let user: User = new User();
    user.firstname = model.firstname;
    user.lastname = model.lastname;
    user.userType = model.userType;
    let addressID: number = model.addressID;
    user.addressID=addressID;

    if(window.confirm(this.msgState.CONFIRM_ADD)){
      this.store.dispatch(new SaveNewUserAction(user));
    }
  }

  tryEgain() {}

  onSaveSuccess() {
    this.router.navigateByUrl("/users-management")
  }
}
