import { UserType } from './../../../../shared/models/user/user.type';
import { User } from 'src/app/shared/models/user/user.model';
import { GetAllAddressesAction } from 'src/app/ngrx/user-s-address/address.action';
import { AddressState } from './../../../../ngrx/user-s-address/address.state';
import { CreateFormUpdateUserAction, UpdateUserAction } from './../../../../ngrx/user/user.actions';
import { UserService } from './../../../../shared/services/user.service';
import { MsgState } from './../../../../ngrx/msg.state';
import { UserState } from './../../../../ngrx/user/user.state';
import { Observable } from 'rxjs';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { Component, OnInit } from '@angular/core';
import { Store } from '@ngrx/store';

@Component({
  selector: 'app-user-update',
  templateUrl: './user-update.component.html',
  styleUrls: ['./user-update.component.css'],
})
export class UserUpdateComponent implements OnInit {
  userID: number;
  constructor(
    activatedRoute: ActivatedRoute,
    private userService: UserService,
    private store: Store<{
      userReducerSelector: UserState;
      addressReducerSelector: AddressState;
    }>,
    private fb: FormBuilder,
    private router: Router
  ) {
    this.userID = activatedRoute.snapshot.params['userID'];
  }

  userState!: UserState;
  userFormGroup!: FormGroup;
  msgState = MsgState;
  addressState!: AddressState;
  userType = UserType;

  ngOnInit(): void {

    this.store.dispatch(new GetAllAddressesAction({}))
    this.store.subscribe( {
      next:(state) =>{
        this.addressState = state.addressReducerSelector;
      },
      error: (err)=>{}
    } );

    this.userService.getUserByID(this.userID).subscribe({
      next: (user) => {
        this.store.dispatch(new CreateFormUpdateUserAction(user, {}));
        this.store.select('userReducerSelector').subscribe({
          next: (state) => {
            this.userState = state;

            this.userFormGroup = this.fb.group({
              userID: [this.userState.currentUser?.userID],
              firstname: [
                this.userState.currentUser?.firstname,
                [
                  Validators.required,
                  Validators.minLength(3),
                  Validators.maxLength(15),
                ],
              ],
              lastname: [
                this.userState.currentUser?.lastname,
                [
                  Validators.required,
                  Validators.minLength(3),
                  Validators.maxLength(15),
                ],
              ],
              userType: [this.userState.currentUser?.userType, Validators.required],
              email: [
                this.userState.currentUser?.email,
                [Validators.required, Validators.email],
              ],
              addressID:[this.userState.currentUser?.address.addressID, Validators.required],
              createdDate:[this.userState.currentUser?.createdDate],
            });
          },
          error: (err) => {},
        });
      },
      error: (err) => {},
    });
  }

  onUserUpdate(){
    let formModel = this.userFormGroup.value;
    let user: User = new User();
    user.userID = formModel.userID;
    user.firstname= formModel.firstname;
    user.lastname = formModel.lastname;
    user.userType = formModel.userType;
    user.email = formModel.email;
    user.createdDate = formModel.createdDate;
    user.addressID = formModel.addressID;

    if(window.confirm(this.msgState.CONFIRM_UPDATE)){
      this.store.dispatch(new UpdateUserAction(user));
    }
  }

  onUpdateOk(){
    this.router.navigateByUrl("/users-management")
  }
}
