import { Address } from './../../../../shared/models/user/address.model';
import { CreateFormUpdateAddressAction, UpdateAddressAction } from './../../../../ngrx/user-s-address/address.action';
import { AddressService } from './../../../../shared/services/address.service';
import { MsgState } from './../../../../ngrx/msg.state';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { AddressState } from './../../../../ngrx/user-s-address/address.state';
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Store } from '@ngrx/store';

@Component({
  selector: 'app-address-update',
  templateUrl: './address-update.component.html',
  styleUrls: ['./address-update.component.css'],
})
export class AddressUpdateComponent implements OnInit {
  addressID: number;

  constructor(
    activatedRoute: ActivatedRoute,
    private addressService: AddressService,
    private store: Store<{ addressReducerSelector: AddressState }>,
    private fb: FormBuilder,
    private router: Router
  ) {
    this.addressID = activatedRoute.snapshot.params['addressID'];
  }

  addressState!: AddressState;
  addressForm!: FormGroup;
  msgState = MsgState;

  ngOnInit(): void {
    this.addressService.getAddressByIDService(this.addressID).subscribe({
      next: (address) => {
        this.store.dispatch(new CreateFormUpdateAddressAction(address));
        this.store.subscribe({
          next: (state) => {
            this.addressState = state.addressReducerSelector;

            this.addressForm = this.fb.group({
              addressID:[ this.addressState.currentAddress?.addressID, Validators.required],
              numHouse: [
                this.addressState.currentAddress?.numHouse,
                [Validators.required, Validators.min(1)],
              ],
              streetName: [
                this.addressState.currentAddress?.streetName,
                [
                  Validators.required,
                  Validators.minLength(5),
                  Validators.maxLength(50),
                ],
              ],
              pb: [
                this.addressState.currentAddress?.pb,
                [Validators.required, Validators.min(1000)],
              ],
              city: [
                this.addressState.currentAddress?.city,
                [
                  Validators.required,
                  Validators.minLength(3),
                  Validators.maxLength(15),
                ],
              ],
              country: [
                this.addressState.currentAddress?.country,
                [
                  Validators.required,
                  Validators.minLength(3),
                  Validators.maxLength(15),
                ],
              ],
            });
          },
        });
      },
      error: (err) => {},
    });
  }

  onAddressUpdate() {
    let addressModel = this.addressForm.value;
    let address: Address = new Address();
    address.addressID = addressModel.addressID;
    address.numHouse = addressModel.numHouse;
    address.streetName = addressModel.streetName;
    address.pb = addressModel.pb;
    address.city = addressModel.city;
    address.country = addressModel.country;

    if(window.confirm(this.msgState.CONFIRM_UPDATE)){
      this.store.dispatch(new UpdateAddressAction(address));
    }
  }

  onUpdateOk(){
    this.router.navigate(['/addresses-management']);
  }
}
