import { catchError } from 'rxjs/operators';
import { Project } from 'src/app/shared/models/project/project.model';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { MyServerErrorHandler } from './serverErrorHandler';
import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';

@Injectable({
  providedIn: 'root',
})
export class ProjectService {
  constructor(
    private httpClient: HttpClient,
    private myServerErrorHandler: MyServerErrorHandler
  ) {}

  createProjectService(project: Project): Observable<Project> {
    let createdProject: Observable<Project> = this.httpClient.post<Project>(
      environment.PROJECTS_SERVER + '/projects/',project);

    return createdProject.pipe(
      catchError(this.myServerErrorHandler.handleError)
    );
  }

  getAllProjectsService(): Observable<Array<Project>> {
    let projects: Observable<Array<Project>> = this.httpClient.get<
      Array<Project>
    >(environment.PROJECTS_SERVER + '/projects');

    return projects.pipe(catchError(this.myServerErrorHandler.handleError));
  }

  getProjectByIDService(projectID: number): Observable<Project> {
    return this.httpClient
      .get<Project>(environment.PROJECTS_SERVER + '/projects/' + projectID)
      .pipe(
        catchError((err) => {
          return this.myServerErrorHandler.handleError(err);
        })
      );
  }

  updateProjectService(
    project: Project
  ): Observable<Project> {
    return this.httpClient.put<Project>(
      environment.PROJECTS_SERVER +
        '/projects/' +
        project.projectID, project);
  }

  deleteProject(projectID: number) {
    return this.httpClient
      .delete(environment.PROJECTS_SERVER + '/projects/' + projectID)
      .pipe(
        catchError((err) => {
          return this.myServerErrorHandler.handleError(err);
        })
      );
  }
}
