import { MyServerErrorHandler } from './serverErrorHandler';
import { catchError } from 'rxjs/operators';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { User } from '../models/user/user.model';
import { Observable } from 'rxjs';
import { HttpClient } from '@angular/common/http';

@Injectable({
  providedIn: 'root',
})
export class UserService {
  constructor(
    private httpClient: HttpClient,
    private serverErrorHandler: MyServerErrorHandler
  ) {}


  geAllUsersService(): Observable<Array<User>> {
    let users: Observable<Array<User>> = this.httpClient.get<User[]>(
      environment.USERS_SERVER + '/users'
    );
    return users;
  }

  createUserService(user: User): Observable<any> {
    return this.httpClient
      .post<any>(environment.USERS_SERVER + '/users/', user)
      .pipe(catchError(this.serverErrorHandler.handleError));
  }

  deleteUserService(userID: number) {
    return this.httpClient
      .delete(environment.USERS_SERVER + '/users/' + userID)
      .pipe(catchError(this.serverErrorHandler.handleError));
  }

  getUserByID(userID: number): Observable<User> {
    return this.httpClient
      .get<User>(environment.USERS_SERVER + '/users/' + userID)
      .pipe(catchError(this.serverErrorHandler.handleError));
  }

  updateUserService(user: User): Observable<User> {
    let updatedUser: Observable<User> = this.httpClient.put<User>(
      environment.USERS_SERVER + '/users/' + user.userID,
      user
    );

    return updatedUser.pipe(catchError(this.serverErrorHandler.handleError));
  }
}
